/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ui.h"
#include "user_callbacks.h"

const int IMAGE_BUF_SIZE = 500;

const char *link_to_sign_up_site = "http://home.openweathermap.org/users/sign_up";

/*
 * List of avalible locations. Names must refer to http://openweathermap.org/ API
 */
const char *city_list[11] = {
    "Seoul", "Warsaw", "London", "Berlin", "Tokyo", "Miami", "Moscow", "Beijing", "Tel_Aviv",
    "Tunis", "Anchorage"
};

/*
 * Buffor for image path
 */
char image_buff[IMAGE_BUF_SIZE];

/*
 * Show XML button callback
 * Function invokes external web browser to open XML
 */
static void _show_raw_xml_cb(void *data, Evas_Object *obj, void *event_info)
{
    /* Store data  */
    appdata_s *ad = data;
    _store_weather_to_file(ad);

    app_control_h control;
    dlog_print(DLOG_DEBUG, LOG_TAG, "> _show_raw_xml_cb");
    app_control_create(&control);
    app_control_set_app_id(control, "org.tizen.browser");
    char path[BUFLEN];
    snprintf(path, BUFLEN, "%s/%s", ad->folder_path, "xml.xml");
    dlog_print(DLOG_DEBUG, LOG_TAG, "> path = %s", path);
    app_control_set_uri(control, path);
    int ret = app_control_send_launch_request(control, NULL, NULL);
    if (ret != APP_CONTROL_ERROR_NONE) {
        dlog_print(DLOG_ERROR, LOG_TAG, "> app_control_send_launch_request()failed"
                   " with error %d. Please make sure org.tizen.browser app is correctly installed.",
                   ret);
    }

    app_control_destroy(control);
}

void _open_sign_up_site(void *data, Evas_Object *obj, void *event_info)
{
    app_control_h control;
    dlog_print(DLOG_DEBUG, LOG_TAG, "> _open_sign_up_site");
    app_control_create(&control);
    app_control_set_app_id(control, "org.tizen.browser");
    app_control_set_uri(control, link_to_sign_up_site);
    int ret = app_control_send_launch_request(control, NULL, NULL);
    if (ret != APP_CONTROL_ERROR_NONE) {
        dlog_print(DLOG_ERROR, LOG_TAG, "> app_control_send_launch_request()failed"
                   " with error %d. Please make sure org.tizen.browser app is correctly installed.",
                   ret);
    }

    app_control_destroy(control);
}

/*
 * Function on image frame click,
 * Makes received XML preview visible
 */
static void _frame_clicked(void *data, Evas_Object *obj, void *event_info)
{
    appdata_s *ad = (appdata_s *)data;
    dlog_print(DLOG_DEBUG, LOG_TAG, "> _frame_clicked");
    elm_win_inwin_activate(ad->xml_popup);
}

/*
 * Function on XML previwe frame click
 * Hides popup window (Evas Inwin)
 */
static void _hide_popup(void *data, Evas_Object *obj, void *event_info)
{
    appdata_s *ad = (appdata_s *)data;
    dlog_print(DLOG_DEBUG, LOG_TAG, "> _hide_popup");
    evas_object_hide(ad->xml_popup);
}

/*
 * Function on selected city change (Evas Spinner)
 * Initiates obtaining and parsing new XML, appropriate for selected location
 */
static void _new_city_cb(void *data, Evas_Object *obj, void *event_info)
{
    appdata_s *ad = (appdata_s *)data;

    /*
     * Check if first invocation
     */
    if (obj != NULL) {
        dlog_print(DLOG_DEBUG, LOG_TAG, "> Value delay changed to %s\n",
                   elm_spinner_special_value_get(obj, elm_spinner_value_get(obj)));

        /*
         * elm_spinner_value_get() - gets number of selected object
         * elm_spinner_special_value_get() - converts number into text
         */
        ad->city = elm_spinner_special_value_get(obj, elm_spinner_value_get(obj));
    } else {
        /*
         * Set on application startup
         */
        ad->city = "Seoul";
    }

    /*
     * Clear displayed weather features list
     */
    elm_list_clear(ad->display_list);

    /*
     * Clear stored weather features
     */
    if (ad->weather_info_list != NULL) {
        Eina_List *l;
        char *str;
        EINA_LIST_FOREACH(ad->weather_info_list, l, str)
        free(str);
        eina_list_free(ad->weather_info_list);
        ad->weather_info_list = NULL;
    }

    /*
     * Get new XML, connected with place
     */
    _get_xml_data(data);
}

/*
 * Create main box, storing other UI elements
 */
void _create_box(appdata_s *ad)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "> Create box");
    ad->main_box = elm_box_add(ad->conform);
    elm_object_content_set(ad->conform, ad->main_box);
    elm_box_horizontal_set(ad->main_box, EINA_FALSE);
    evas_object_show(ad->main_box);
}

/*
 * Create spinner, city choose element
 */
void _create_spinner(appdata_s *ad)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "> Create spinner");

    ad->spinner = elm_spinner_add(ad->conform);
    evas_object_size_hint_align_set(ad->spinner, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(ad->spinner, EVAS_HINT_EXPAND, 0.03);
    elm_box_pack_end(ad->main_box, ad->spinner);
    evas_object_show(ad->spinner);
    elm_spinner_editable_set(ad->spinner, EINA_FALSE);
    int size = sizeof(city_list) / sizeof(char *);
    elm_spinner_min_max_set(ad->spinner, 1, size);

    /*
     * Add all cities from list to spinner
     */
    for (int i = 0; i < size; i++)
        elm_spinner_special_value_add(ad->spinner, i + 1, city_list[i]);

    evas_object_smart_callback_add(ad->spinner, "changed", _new_city_cb, ad);
}

/*
 * Create received XML popup
 */
void _create_raw_text(appdata_s *ad)
{

    ad->xml_popup = elm_win_inwin_add(ad->win);

    /*
     * Add scrolelr to display long entry box
     */
    Evas_Object *scroller = elm_scroller_add(ad->win);
    evas_object_size_hint_align_set(scroller, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, 0.2);
    evas_object_show(scroller);
    elm_scroller_bounce_set(scroller, EINA_TRUE, EINA_FALSE);
    elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_ON, ELM_SCROLLER_POLICY_ON);
    elm_scroller_propagate_events_set(scroller, EINA_TRUE);

    /*
     * Create entry to display text
     */
    ad->entry = elm_entry_add(ad->conform);
    elm_object_text_set(ad->entry, "Please enable Internet connection");
    elm_entry_single_line_set(ad->entry, EINA_FALSE);
    evas_object_size_hint_align_set(ad->entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(ad->entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_show(ad->entry);
    elm_entry_input_panel_enabled_set(ad->entry, EINA_FALSE);
    elm_object_content_set(scroller, ad->entry);

    /*
     * Register hide on click callback
     */
    evas_object_smart_callback_add(ad->entry, "clicked", _hide_popup, ad);

    elm_win_inwin_content_set(ad->xml_popup, scroller);
}

void _create_data_list(appdata_s *ad)
{
    /*
     * Create list for weather features display
     */
    ad->display_list = elm_list_add(ad->win);
    evas_object_size_hint_align_set(ad->display_list, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(ad->display_list, EVAS_HINT_EXPAND, 0.6);
    elm_box_pack_end(ad->main_box, ad->display_list);
    evas_object_show(ad->display_list);

    /*
     * Set city, Display features
     */
    _new_city_cb(ad, NULL, NULL);
}

/*
 * Creates box with background
 */
Evas_Object *my_frame_add(Evas_Object *parent, Evas_Object *in_box, const char *text)
{
    /*
     * Create table
     */
    Evas_Object *object = elm_table_add(parent);
    elm_table_homogeneous_set(object, EINA_TRUE);
    evas_object_size_hint_weight_set(object, EVAS_HINT_EXPAND, 0.0);
    evas_object_size_hint_align_set(object, EVAS_HINT_FILL, 0.0);
    evas_object_show(object);

    Evas_Object *background = elm_bg_add(object);
    elm_bg_color_set(background, 240, 200, 180);

    evas_object_size_hint_weight_set(in_box, EVAS_HINT_EXPAND, 0.0);
    evas_object_size_hint_align_set(in_box, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_show(in_box);

    elm_table_pack(object, background, 0, 0, 1, 1);
    elm_table_pack(object, in_box, 0, 0, 1, 1);

    /*
     * Create label
     */
    Evas_Object *in_label = elm_label_add(object);
    char label_text[100];
    snprintf(label_text, 100, "<font_size=16>%s</font>", text);
    elm_object_text_set(in_label, label_text);
    evas_object_size_hint_align_set(in_label, 0.0, 0.0);
    evas_object_show(in_label);
    evas_object_size_hint_padding_set(background, 2, 1, 2, 2);
    evas_object_size_hint_padding_set(in_box, 0, 0, 30, 5);
    evas_object_size_hint_padding_set(in_label, 10, 0, 5, 0);
    elm_table_pack(object, in_label, 0, 0, 1, 1);

    evas_object_size_hint_align_set(background, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(background, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_show(background);

    return object;
}

/*
 * Create box for image display
 */
void _create_image_box(appdata_s *ad)
{
    /*
     * Get path for application resource directory
     */
    char *res_path = app_get_resource_path();
    dlog_print(DLOG_DEBUG, LOG_TAG, "> _create_image_box %s ", res_path);

    /*
     * Add image
     */
    ad->image = elm_image_add(ad->win);
    evas_object_size_hint_align_set(ad->image, 0.5, 0);
    evas_object_size_hint_weight_set(ad->image, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_show(ad->image);

    /*
     * Add frame storing image
     */
    Evas_Object *fr = my_frame_add(ad->win, ad->image, "Click for XML");
    evas_object_size_hint_align_set(fr, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(fr, EVAS_HINT_EXPAND, 0);
    elm_box_pack_end(ad->main_box, fr);
    evas_object_show(fr);

    /*
     * Display raw recived XML on frame click
     */
    evas_object_smart_callback_add(fr, "clicked", _frame_clicked, ad);

    /*
     * Set image path
     */
    snprintf(image_buff, IMAGE_BUF_SIZE, "%s%s", res_path, "internet.png");
    image_buff[IMAGE_BUF_SIZE - 1] = 0x00;

    if (!elm_image_file_set(ad->image, image_buff, NULL)) {
        dlog_print(DLOG_ERROR, LOG_TAG, "error: could not load image \"%s\"", image_buff);
        return;
    }

    /*
     * Format image
     */
    elm_image_no_scale_set(ad->image, TRUE);
    elm_image_resizable_set(ad->image, EINA_TRUE, EINA_TRUE);
    elm_image_smooth_set(ad->image, EINA_FALSE);
    elm_image_aspect_fixed_set(ad->image, EINA_FALSE);
    elm_image_fill_outside_set(ad->image, EINA_FALSE);
    elm_image_editable_set(ad->image, EINA_FALSE);

    /*
     * Display raw received XML on image click
     */
    evas_object_smart_callback_add(ad->image, "clicked", _frame_clicked, ad);

    free(res_path);
}

/*
 * Set image appropriate to actual weather
 */
void _set_weather_image(float temp, int humidity, appdata_s *ad)
{
    dlog_print(DLOG_DEBUG, LOG_TAG, "> set_weather_image %f %d", temp, humidity);
    char *windicator = NULL;
    char *res_path = app_get_resource_path();

    if (temp < 0)
        windicator = "snow.png";
    else if (humidity > 75)
        windicator = "rain.png";
    else if (temp > 20)
        windicator = "sun.png";
    else if (temp > 12)
        windicator = "sc.png";
    else
        windicator = "clouds.png";

    dlog_print(DLOG_DEBUG, LOG_TAG, "> %s", windicator);
    snprintf(image_buff, IMAGE_BUF_SIZE, "%s%s", res_path, windicator);
    image_buff[IMAGE_BUF_SIZE - 1] = 0x00;
    free(res_path);

    /*
     * Set image file
     */
    if (!elm_image_file_set(ad->image, image_buff, NULL))
        dlog_print(DLOG_ERROR, LOG_TAG, "error: could not load image \"%s\"", image_buff);
}

/*
 * Create bottom buttons
 */
void _prepare_ui(appdata_s *ad)
{
    ad->city = "Seoul";
    ad->xml_to_str = NULL;
}

void _create_buttons(appdata_s *ad)
{
    Evas_Object *btprev = elm_button_add(ad->main_box);
    evas_object_size_hint_align_set(btprev, EVAS_HINT_FILL, 1.0);
    evas_object_size_hint_weight_set(btprev, EVAS_HINT_EXPAND, 0.0);
    elm_object_text_set(btprev, "Show XML");
    evas_object_smart_callback_add(btprev, "clicked", _show_raw_xml_cb, ad);
    elm_box_pack_end(ad->main_box, btprev);
    evas_object_show(btprev);
}
